"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.hasSpread = hasSpread;
exports.hasSkip = hasSkip;
exports.toAttrsArray = toAttrsArray;
exports.toAttrsCalls = toAttrsCalls;

var _spreadAttribute = _interopRequireDefault(require("./runtime/spread-attribute"));

var _toFunctionCall = _interopRequireDefault(require("./ast/to-function-call"));

var _idomMethod = _interopRequireDefault(require("./idom-method"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

// Detects if one of the attributes is a JSX Spread Attribute
function hasSpread(attributes) {
  return attributes.some(function (attr) {
    return attr.isJSXSpreadAttribute();
  });
} // Detects if the skip attribute is used


function hasSkip(attributes, _ref) {
  var opts = _ref.opts;
  var skipAttribute = opts.skipAttribute || "__skip";
  return attributes.some(function (attr) {
    return attr.isJSXAttribute() && attr.node.name.name === skipAttribute;
  });
} // Returns an array of `name`-`value` attribute pairs


function toAttrsArray(attrs) {
  var pairsArray = [];
  attrs.forEach(function (_ref2) {
    var name = _ref2.name,
        value = _ref2.value;
    pairsArray.push(name);
    pairsArray.push(value);
  });
  return pairsArray;
} // Returns an array of iDOM `attr` calls


function toAttrsCalls(attrs, plugin) {
  var attr = (0, _idomMethod["default"])("attr", plugin);
  var spreadAttr = (0, _spreadAttribute["default"])(plugin);
  return attrs.map(function (_ref3) {
    var name = _ref3.name,
        value = _ref3.value,
        isSpread = _ref3.isSpread;

    if (isSpread) {
      return (0, _toFunctionCall["default"])(spreadAttr, [value]);
    }

    return (0, _toFunctionCall["default"])(attr, [name, value]);
  });
}