"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = buildChildren;

var _cleanText = _interopRequireDefault(require("./clean-text"));

var _toFunctionCall = _interopRequireDefault(require("./ast/to-function-call"));

var _renderArbitrary = _interopRequireDefault(require("./runtime/render-arbitrary"));

var _idomMethod = _interopRequireDefault(require("./idom-method"));

var _isLiteralOrSpecial = _interopRequireDefault(require("./is-literal-or-special"));

var _toString = _interopRequireDefault(require("./ast/to-string"));

var _completionRecords = require("./completion-records");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

// String concatenations are special cased, so template literals don't
// require a call to renderArbitrary.
function isStringConcatenation(path) {
  if (!(path.isBinaryExpression() && path.node.operator === "+")) {
    return false;
  }

  var left = path.get("left");
  var right = path.get("right");
  return left.isStringLiteral() || right.isStringLiteral() || isStringConcatenation(left) || isStringConcatenation(right);
} // Transforms the children into an array of iDOM function calls


function buildChildren(children, plugin) {
  var replacedElements = plugin.replacedElements,
      fastRoots = plugin.fastRoots;
  children = children.reduce(function (children, child) {
    var wasInExpressionContainer = child.isJSXExpressionContainer();

    if (wasInExpressionContainer) {
      child = child.get("expression");
    }

    if (child.isJSXEmptyExpression()) {
      return children;
    }

    var sequence;

    while (child.isSequenceExpression() && !replacedElements.has(child)) {
      sequence = child;
      var expressions = child.get("expressions");
      var i = void 0;

      for (i = 0; i < expressions.length - 1; i++) {
        children.push(expressions[i].node);
      }

      child = expressions[i];
    }

    var _child = child,
        node = _child.node;

    if (child.isJSXText() || (0, _isLiteralOrSpecial["default"])(child)) {
      var value; // Clean up the text, so we don't have to have multiple TEXT nodes.

      if (child.isJSXText()) {
        value = (0, _cleanText["default"])(child);
      } else {
        value = (0, _toString["default"])(child, true);
      } // Only strings and numbers will print, anything else is skipped.


      if (!value) {
        return children;
      }

      node = (0, _toFunctionCall["default"])((0, _idomMethod["default"])("text", plugin), [value]);
    } else if (isStringConcatenation(child)) {
      node = (0, _toFunctionCall["default"])((0, _idomMethod["default"])("text", plugin), [node]);
    } else if (wasInExpressionContainer) {
      var element = sequence || child;

      if (!replacedElements.has(child) && !fastRoots.has(element)) {
        var completions = (0, _completionRecords.getCompletionRecords)(child);
        var nonJSX = completions.some(function (c) {
          return !replacedElements.has(c);
        });

        if (nonJSX) {
          // Arbitrary expressions, e.g. variables, need to be inspected at runtime
          // to determine how to render them.
          node = (0, _toFunctionCall["default"])((0, _renderArbitrary["default"])(plugin), [node]);
        }
      }
    }

    children.push(node);
    return children;
  }, []);
  return children;
}