"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getCompletionRecords = getCompletionRecords;
exports.isCompletionRecord = isCompletionRecord;

var _fastRoot = _interopRequireWildcard(require("./fast-root"));

var _last = _interopRequireDefault(require("./last"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; if (obj != null) { var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var map = new WeakMap();

function getCompletionRecords(path) {
  var find = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
  var stack = [path];
  var i = 0;

  while (i < stack.length) {
    var p = stack[i];

    if (p.isSequenceExpression()) {
      stack[i] = (0, _last["default"])(p.get("expressions"));
    } else if (p.isConditionalExpression()) {
      stack[i] = p.get("consequent");
      stack.push(p.get("alternate"));
    } else if (p.isLogicalExpression()) {
      if (!find || p.node.operator === "||") {
        stack.push(p.get("left"));
      }

      stack[i] = p.get("right");
    } else if (find && p === find) {
      return true;
    } else {
      i++;
    }
  }

  return find ? false : stack;
} // Determines if the element is a completion record.


function isCompletionRecord(path, _ref) {
  var fastRoots = _ref.fastRoots,
      opts = _ref.opts;
  var is;

  if ((0, _fastRoot["default"])(path, opts) && (0, _fastRoot.fastRootAncestor)(path, fastRoots)) {
    is = true;
  } else {
    var container = path.findParent(function (p) {
      return p.isJSX();
    });

    if (!container || container.isJSXElement()) {
      is = true;
    } else if (container.isJSXAttribute() || container.parentPath.isJSXAttribute()) {
      is = false;
    } else {
      is = getCompletionRecords(container.get("expression"), path);
    }
  }

  map.set(path, is);
  return is;
}