"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = elementOpenCall;

var _toFunctionCall = _interopRequireDefault(require("./ast/to-function-call"));

var _toReference = _interopRequireDefault(require("./ast/to-reference"));

var _idomMethod = _interopRequireDefault(require("./idom-method"));

var _isComponent = _interopRequireDefault(require("./is-component"));

var _extractOpenArguments2 = _interopRequireDefault(require("./extract-open-arguments"));

var _attributes = require("./attributes");

var t = _interopRequireWildcard(require("@babel/types"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; if (obj != null) { var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

// Returns the opening element's function call.
function elementOpenCall(path, plugin) {
  var name = path.get("name");
  var attributes = path.get("attributes");
  var useReference = (0, _isComponent["default"])(name, plugin);
  var tag = (0, _toReference["default"])(name.node, useReference);
  var args = [tag];

  var _extractOpenArguments = (0, _extractOpenArguments2["default"])(path, plugin),
      key = _extractOpenArguments.key,
      statics = _extractOpenArguments.statics,
      attrs = _extractOpenArguments.attrs; // Only push arguments if they're needed


  if (key || statics) {
    args.push(key || t.nullLiteral());
  }

  if (statics) {
    args.push(statics);
  } // If there is a spread element, we need to use
  // the elementOpenStart/elementOpenEnd syntax.
  // This allows spreads to be transformed into
  // attr(name, value) calls.


  if ((0, _attributes.hasSpread)(attributes)) {
    var expressions = [(0, _toFunctionCall["default"])((0, _idomMethod["default"])("elementOpenStart", plugin), args)].concat(_toConsumableArray((0, _attributes.toAttrsCalls)(attrs, plugin)), [(0, _toFunctionCall["default"])((0, _idomMethod["default"])("elementOpenEnd", plugin), [tag])]);
    return t.sequenceExpression(expressions);
  }

  if (attrs) {
    // Only push key and statics if they have not
    // already been pushed.
    if (!statics) {
      if (!key) {
        args.push(t.nullLiteral());
      }

      args.push(t.nullLiteral());
    }

    args.push.apply(args, _toConsumableArray((0, _attributes.toAttrsArray)(attrs)));
  }

  var selfClosing = path.node.selfClosing;
  var elementFunction = selfClosing && !(0, _attributes.hasSkip)(attributes, plugin) ? "elementVoid" : "elementOpen";
  return (0, _toFunctionCall["default"])((0, _idomMethod["default"])(elementFunction, plugin), args);
}