"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _isLiteralOrSpecial = _interopRequireWildcard(require("./is-literal-or-special"));

var _last = _interopRequireDefault(require("./last"));

var _completionRecords = require("./completion-records");

var t = _interopRequireWildcard(require("@babel/types"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; if (obj != null) { var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function addClosureVar(expression, closureVars, defaultName) {
  var init = expression.node;
  var id = expression.scope.generateUidIdentifierBasedOnNode(init, defaultName);
  closureVars.push({
    id: id,
    init: init
  });
  return id;
} // Extracts variable expressions into an array of closure parameters,
// so that when the closure is finally evaluated, it will have the correct
// values.


var expressionExtractor = {
  JSXSpreadAttribute: function JSXSpreadAttribute(path) {
    var closureVarsStack = this.closureVarsStack;
    var argument = path.get("argument");
    var id = addClosureVar(argument, (0, _last["default"])(closureVarsStack), "spread");
    argument.replaceWith(id);
  },
  JSXExpressionContainer: function JSXExpressionContainer(path) {
    var expression = path.get("expression"); // If the variable is constant (or will be wrapped), don't extract.

    if ((0, _isLiteralOrSpecial["default"])(expression) || expression.isJSXElement()) {
      return;
    }

    var closureVars = (0, _last["default"])(this.closureVarsStack);
    var parentPath = path.parentPath,
        scope = path.scope; // Only call defer JSX Children Expressions

    if (!parentPath.isJSXElement()) {
      expression.replaceWith(addClosureVar(expression, closureVars, parentPath.node.name.name));
      return;
    }

    var completions = (0, _completionRecords.getCompletionRecords)(expression);
    var deferred = completions.filter(function (c) {
      // `fn() && other` and `fn() || other` cannot be deferred.
      return c.key !== "left" && c.isCallExpression();
    }); // Exit early if there's nothing to defer.

    if (deferred.length === 0) {
      expression.replaceWith(addClosureVar(expression, closureVars, "child"));
      return;
    }

    var everyBranchHasCall = deferred.length >= completions.length;
    var deferredId = scope.generateUidIdentifier("deferred");
    var argId;
    var branchId;

    if (completions.length > 1) {
      branchId = scope.generateUidIdentifier("b");
      scope.push({
        id: branchId,
        init: t.numericLiteral(0)
      });
    } // Map our deferred calls into their important information


    var calls = deferred.map(function (call, i) {
      var node = call.node;
      var callee = call.get("callee");
      var isMemberExpression = callee.isMemberExpression(); // Transform our calls into their context. For normal calls, it's just the
      // function reference. For member expressions (`obj.fn()`), it's the `obj`.
      // We'll invoke the function properly later on.

      var context = isMemberExpression ? callee.node.object : callee.node;

      if (branchId) {
        context = t.sequenceExpression([t.assignmentExpression("=", branchId, t.numericLiteral(i + 1)), context]);
      }

      call.replaceWith(context);
      return {
        node: node,
        isMemberExpression: isMemberExpression
      };
    }); // Now push the transformed expression into our closure variables.
    // This node has all the deferrable calls remapped into their contexts,
    // and won't actually invoke them.

    closureVars.push({
      id: deferredId,
      init: expression.node
    }); // Now, push the branching identifier.

    if (branchId) {
      closureVars.push({
        id: branchId,
        init: branchId
      });
    } // Finally, transform the calls into their evaluted "contex" form.


    var evaluatedBranches = calls.map(function (struct) {
      var node = struct.node,
          isMemberExpression = struct.isMemberExpression; // This reverses the context transform done earlier.

      if (isMemberExpression) {
        node.callee.object = deferredId;
      } else {
        node.callee = deferredId;
      } // Any arguments are now passed through the evaluated args array.


      var index = 0;
      var args = [];
      var length = node.arguments.reduce(function (length, arg) {
        if ((0, _isLiteralOrSpecial.isLiteralOrSpecialNode)(arg) || t.isJSXElement(arg)) {
          return length;
        }

        return length + 1;
      }, 0);
      node.arguments = node.arguments.map(function (arg) {
        if ((0, _isLiteralOrSpecial.isLiteralOrSpecialNode)(arg) || t.isJSXElement(arg)) {
          return arg;
        }

        var spread = t.isSpreadElement(arg);
        args.push(spread ? arg.argument : arg);

        if (!argId) {
          argId = scope.generateUidIdentifier("args");
        }

        var evaluated;

        if (length === 1) {
          evaluated = argId;
        } else {
          evaluated = t.memberExpression(argId, t.numericLiteral(index++), true);
        }

        return spread ? t.spreadElement(evaluated) : evaluated;
      });
      return {
        node: node,
        args: length === 0 ? null : length === 1 ? args[0] : t.arrayExpression(args)
      };
    });
    var evaluated = everyBranchHasCall ? null : deferredId;
    var evaluatedArgs = t.nullLiteral();

    for (var i = evaluatedBranches.length - 1; i >= 0; i--) {
      var _evaluatedBranches$i = evaluatedBranches[i],
          node = _evaluatedBranches$i.node,
          args = _evaluatedBranches$i.args;

      if (args) {
        if (branchId) {
          evaluatedArgs = t.conditionalExpression(t.binaryExpression("==", branchId, t.numericLiteral(i + 1)), args, evaluatedArgs);
        } else {
          evaluatedArgs = args;
        }
      }

      if (evaluated) {
        evaluated = t.conditionalExpression(t.binaryExpression("==", branchId, t.numericLiteral(i + 1)), node, evaluated);
      } else {
        evaluated = node;
      }
    }

    if (argId) {
      closureVars.push({
        id: argId,
        init: evaluatedArgs
      });
    }

    expression.replaceWith(evaluated);
  }
};
var _default = expressionExtractor;
exports["default"] = _default;