"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = extractOpenArguments;

var _isLiteralOrSpecial = _interopRequireDefault(require("./is-literal-or-special"));

var _hoist = require("./hoist");

var _uuid = _interopRequireDefault(require("./uuid"));

var _toString = _interopRequireDefault(require("./ast/to-string"));

var _last = _interopRequireDefault(require("./last"));

var t = _interopRequireWildcard(require("@babel/types"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; if (obj != null) { var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

// Extracts attributes into the appropriate
// attribute array. Static attributes and the key
// are placed into static attributes, and expressions
// are placed into the variadic attributes.
function extractOpenArguments(path, plugin) {
  var attributes = path.get("attributes");
  var elementVarsStack = plugin.elementVarsStack,
      closureVarsStack = plugin.closureVarsStack;
  var requireStaticsKey = plugin.opts.requireStaticsKey;
  var elementVars = (0, _last["default"])(elementVarsStack);
  var closureVars = (0, _last["default"])(closureVarsStack) || [];
  var attrs = [];
  var key = null;
  var keyIndex = -1;
  attributes.forEach(function (attribute) {
    if (attribute.isJSXSpreadAttribute()) {
      attrs.push({
        name: null,
        value: attribute.get("argument").node,
        isSpread: true,
        isStatic: false
      });
      return;
    }

    var namePath = attribute.get("name");
    var name;

    if (namePath.isJSXIdentifier()) {
      name = t.stringLiteral(namePath.node.name);
    } else {
      name = t.stringLiteral("".concat(namePath.node.namespace.name, ":").concat(namePath.node.name.name));
    }

    var value = attribute.get("value");
    var _value = value,
        node = _value.node; // Attributes without a value are interpreted as `true`.

    if (!node) {
      value.replaceWith(t.jSXExpressionContainer(t.booleanLiteral(true)));
    } // Get the value inside the expression.


    if (value.isJSXExpressionContainer()) {
      value = value.get("expression");
      node = value.node;
    }

    var literal = (0, _isLiteralOrSpecial["default"])(value);

    if (literal) {
      node = (0, _toString["default"])(value);
    } // The key attribute must be passed to the `elementOpen` call.


    if (name.value === "key") {
      var _value2 = value,
          scope = _value2.scope; // If it's not a literal key, we must assign it in the statics array.

      if (!literal) {
        // Make sure the rearranging the key for iDOM's call does not affect
        // the value of a previous attribute.
        attrs.forEach(function (attr) {
          var name = attr.name,
              value = attr.value,
              isStatic = attr.isStatic;

          if (isStatic) {
            return;
          } // Closure vars don't need to considered, they're already evaluated properly.


          if (t.isIdentifier(value) && closureVars.find(function (_ref) {
            var id = _ref.id;
            return id === value;
          })) {
            return;
          }

          var id = scope.generateDeclaredUidIdentifier(name ? name.value : "spread");
          elementVars.push(t.assignmentExpression("=", id, value));
          attr.value = id;
        });

        if (!value.isIdentifier()) {
          var id = (0, _hoist.generateHoistName)(path, "key");
          scope.push({
            id: id
          });
          elementVars.push(t.assignmentExpression("=", id, node));
          node = id;
        }

        keyIndex = attrs.reduce(function (sum, _ref2) {
          var isStatic = _ref2.isStatic;
          return sum + (isStatic ? 2 : 0);
        }, 1);
        literal = true;
      }

      key = node;
    }

    attrs.push({
      name: name,
      value: node,
      isSpread: false,
      isStatic: literal
    });
  });
  var statics = null;
  var hasStatic = !!attrs.find(function (_ref3) {
    var isStatic = _ref3.isStatic;
    return isStatic;
  });

  if (hasStatic && !key && !requireStaticsKey) {
    // Generate a UUID to be used as the key.
    key = t.stringLiteral((0, _uuid["default"])(plugin));
  }

  if (hasStatic && key) {
    var staticAttrs = [];
    statics = (0, _hoist.generateHoistName)(path, "statics");
    attrs = attrs.filter(function (attr) {
      if (attr.isStatic) {
        staticAttrs.push(attr.name, attr.value);
        return false;
      }

      return true;
    });
    (0, _hoist.addHoistedDeclarator)(path.scope, statics, t.arrayExpression(staticAttrs), plugin);

    if (keyIndex > -1) {
      var keyAssignment = (0, _last["default"])(elementVars);
      var staticsAssignment = t.assignmentExpression("=", t.memberExpression(statics, t.numericLiteral(keyIndex), true), keyAssignment ? keyAssignment.right : key);
      staticAttrs[keyIndex] = t.stringLiteral("");

      if (keyAssignment) {
        keyAssignment.right = staticsAssignment;
      } else {
        elementVars.push(staticsAssignment);
      }
    }
  }

  if (attrs.length === 0) {
    attrs = null;
  }

  return {
    key: key,
    statics: statics,
    attrs: attrs
  };
}