"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useFastRoot;
exports.fastRootAncestor = fastRootAncestor;

var _last = _interopRequireDefault(require("./last"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

// Determines if this JSX element has fast-root enabled or disabled.
// Fast root may be explicitly set using inline comments, and defaults
// to the global option value.
function useFastRoot(path, _ref) {
  var _ref$fastRoot = _ref.fastRoot,
      fastRoot = _ref$fastRoot === void 0 ? false : _ref$fastRoot;
  path.find(function (path) {
    var comments = path.node.leadingComments;
    return comments && comments.find(function (comment) {
      var match = /@incremental-dom.+(enable|disable)-fastRoot/.exec(comment.value);

      if (match) {
        fastRoot = match[1] === "enable";
        return true;
      }
    });
  });
  return fastRoot;
} // Find the root ancestor element of a fast-root enabled element.


function fastRootAncestor(path, fastRoots) {
  var ancestor = path;
  var root = path;
  var child = path;

  while (child = ancestor, ancestor = ancestor.parentPath) {
    if (ancestor.isJSXElement()) {
      // Found!
      break;
    }

    if (ancestor.isJSXAttribute()) {
      // JSX Attributes can never be roots.
      return false;
    }

    if (ancestor.isJSXExpressionContainer()) {
      // Don't muck up the root. We're interested in the expression, not the
      // container.
      continue;
    }

    if (ancestor.isSequenceExpression()) {
      // If we didn't traverse up from the child expression, we're not really
      // a child.
      if ((0, _last["default"])(ancestor.get("expressions")) !== child) {
        return false;
      }
    } else if (ancestor.isConditionalExpression()) {
      if (child.key === "test") {
        return false;
      } // Break root detection chain. We could extend this to determine if both
      // branches are actually JSX elements (or fast-root containers).


      continue;
    } else if (ancestor.isLogicalExpression()) {
      if (child.key === "left" && ancestor.node.operator === "&&") {
        // Since JSX elements are always truthy, this guarantees the branching
        // will pass this element and continue after the &&.
        return false;
      } // Break root detection chain. We could extend this to determine if both
      // branches are actually JSX elements (or fast-root containers).


      continue;
    } // If we haven't broken root detection chain yet, update the root.


    if (child === root) {
      root = ancestor;
    }
  }

  if (root) {
    fastRoots.add(root);
  }

  return true;
}