"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setupHoists = setupHoists;
exports.hoist = hoist;
exports.addHoistedDeclarator = addHoistedDeclarator;
exports.generateHoistName = generateHoistName;

var t = _interopRequireWildcard(require("@babel/types"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; if (obj != null) { var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var namespace = "incremental-dom-hoists"; // Sets up the file to hoist all statics

function setupHoists(_ref) {
  var file = _ref.file;
  // A map to store helper variable references
  // for each file
  file.set(namespace, []);
}

function hoist(program, _ref2) {
  var file = _ref2.file;
  var hoists = file.get(namespace);

  if (hoists.length) {
    var declaration = t.variableDeclaration("const", hoists);
    program.unshiftContainer("body", declaration);
  }
} // Hoists the variable to the top of the file.


function addHoistedDeclarator(scope, ref, value, _ref3) {
  var file = _ref3.file;
  var declarator = t.variableDeclarator(ref, value);
  file.get(namespace).push(declarator);
  return ref;
} // Name Smartly Do Good.


function generateHoistName(path) {
  var suffix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "ref";
  var parent = path.findParent(function (p) {
    return p.isVariableDeclarator() || p.isAssignmentExpression() || p.isStatement();
  });
  var name;

  if (parent.isAssignmentExpression()) {
    name = parent.get("left");
  } else if (parent.isVariableDeclarator()) {
    name = parent.get("id");
  } else {
    name = path.get("name");
  }

  return generateHoistNameBasedOn(name, suffix);
} // Name Smartly Do Good.


function generateHoistNameBasedOn(path) {
  var suffix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "";
  var names = [];

  if (suffix) {
    names.push(suffix);
  }

  var current = path;

  while (!(current.isIdentifier() || current.isJSXIdentifier())) {
    names.push(current.node.property.name);
    current = current.get("object");
  }

  names.push(current.node.name);
  return path.scope.generateUidIdentifier(names.reverse().join("$"));
}