"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = patchRoots;

var _collectImports = _interopRequireDefault(require("./collect-imports"));

var _moduleSource = _interopRequireDefault(require("./module-source"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var patchRootsMap = new WeakMap(); // Gathers the function nodes that are passed into the iDOM `patch` function.

function patchRoots(plugin) {
  var file = plugin.file;
  var patchRoots = patchRootsMap.get(file);

  if (!patchRoots) {
    var importSource = (0, _moduleSource["default"])(plugin);

    if (!importSource) {
      patchRootsMap.set(file, []);
      return [];
    }

    var patchCalls = [];
    var imports = (0, _collectImports["default"])(file.path);
    var iDOMImport = imports.find(function (imported) {
      return imported.source === importSource;
    }); // Gather all the patch calls

    if (iDOMImport && iDOMImport.imported.indexOf("patch") > -1) {
      var patchImport = iDOMImport.specifiers.find(function (imported) {
        return imported.imported === "patch";
      });
      var binding = file.scope.getBinding(patchImport.local);
      binding.referencePaths.forEach(function (reference) {
        var parentPath = reference.parentPath;

        if (parentPath.isCallExpression()) {
          patchCalls.push(parentPath);
        }
      });
    } else if (iDOMImport && iDOMImport.imported.indexOf("*") > -1) {
      var starImport = iDOMImport.specifiers.find(function (imported) {
        return imported.kind === "namespace";
      });

      var _binding = file.scope.getBinding(starImport.local);

      _binding.referencePaths.forEach(function (reference) {
        var parentPath = reference.parentPath;

        if (parentPath.isMemberExpression()) {
          var property = parentPath.get("property");

          if (property.isIdentifier({
            name: "patch"
          })) {
            var grandParentPath = parentPath.parentPath;

            if (grandParentPath.isCallExpression()) {
              patchCalls.push(grandParentPath);
            }
          }
        }
      });
    } // Now, gather the renderer function that is the second param.


    patchRoots = patchCalls.reduce(function (roots, call) {
      var renderer = call.get("arguments.1");

      if (!renderer) {
        return roots;
      }

      if (renderer.isFunction()) {
        roots.push(renderer);
      } else if (renderer.isIdentifier()) {
        var name = renderer.node.name;

        var _binding2 = renderer.scope.getBinding(name);

        if (_binding2) {
          roots.push(_binding2.path);
        }
      }

      return roots;
    }, []);
    patchRootsMap.set(file, patchRoots);
  }

  return patchRoots;
}