"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isRootOrDescendant = isRootOrDescendant;
exports["default"] = rootJSX;

var _isReturned = _interopRequireDefault(require("./is-returned"));

var _fastRoot = _interopRequireDefault(require("./fast-root"));

var _patchRoots = _interopRequireDefault(require("./patch-roots"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var rootMap = new WeakMap();
var rootElementFinder = {
  JSXElement: function JSXElement(path) {
    // We're looking for a root element, which must be returned by the function.
    if ((0, _isReturned["default"])(path) && inPatchRoot(path, this)) {
      this.root = path;
      path.stop();
    }
  },
  // Don't traverse into other functions, since they cannot contain the root.
  Function: function Function(path) {
    path.skip();
  }
}; // Determines if the JSX element is nested inside a patch's rendering function.

function inPatchRoot(path, plugin) {
  var opts = plugin.opts;

  if ((0, _fastRoot["default"])(path, opts)) {
    return true;
  }

  var roots = (0, _patchRoots["default"])(plugin);
  return !roots.length || path.findParent(function (parent) {
    return roots.indexOf(parent) > -1;
  });
} // Determines if this element is the root element or descends directly from it.


function isRootOrDescendant(path, plugin) {
  var parent = path.getFunctionParent();
  var root = rootJSX(path, plugin);

  if (!root) {
    return false;
  }

  var enclosing = path.findParent(function (path) {
    return path.isJSXElement() || path.isJSXAttribute();
  });

  if (enclosing) {
    if (enclosing.isJSXAttribute()) {
      // Attributes cannot contain a root.
      return false;
    } else {
      // Technically, not true. But we consider any element that directly
      // descends from another to be in a "root". The actual ancestor element
      // may eventually be wrapped if it's not a root.
      return true;
    }
  }

  if (root === path) {
    return true;
  }

  if (root.getFunctionParent() !== parent) {
    return false;
  } // We can have multiple roots at the same level.


  return (0, _isReturned["default"])(path);
} // Finds the root JSX Element in this scope an all higher scopes.


function rootJSX(path, plugin) {
  if (rootMap.has(path)) {
    return rootMap.get(path);
  }

  var state = Object.assign({}, plugin, {
    root: null
  });
  var root;
  var top = path.find(function (path) {
    if (path.isFunction() || path.isProgram()) {
      state.root = null;

      if (rootMap.has(path)) {
        root = rootMap.get(path);
        return true;
      } else {
        path.traverse(rootElementFinder, state);

        if (state.root) {
          root = state.root;
        }

        rootMap.set(path, state.root);
      }
    }
  });

  if (root) {
    var stop = top || root.getFunctionParent();
    path.find(function (path) {
      if (path === stop) {
        return true;
      }

      if (path.isFunction() || path.isProgram()) {
        rootMap.set(path, root);
      }
    });
  }

  return root;
}