"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = _default;

var _rootJsx = _interopRequireWildcard(require("./helpers/root-jsx"));

var _inject = require("./helpers/inject");

var _hoist = require("./helpers/hoist");

var _completionRecords = require("./helpers/completion-records");

var _extractExpressions = _interopRequireDefault(require("./helpers/extract-expressions"));

var _inlineExpressions = _interopRequireDefault(require("./helpers/inline-expressions"));

var _jsxWrapper = _interopRequireDefault(require("./helpers/runtime/jsx-wrapper"));

var _toFunctionCall = _interopRequireDefault(require("./helpers/ast/to-function-call"));

var _idomMethod = _interopRequireDefault(require("./helpers/idom-method"));

var _flattenExpressions = _interopRequireDefault(require("./helpers/ast/flatten-expressions"));

var _statementsWithReturnLast = _interopRequireDefault(require("./helpers/ast/statements-with-return-last"));

var _elementOpenCall = _interopRequireDefault(require("./helpers/element-open-call"));

var _elementCloseCall = _interopRequireDefault(require("./helpers/element-close-call"));

var _buildChildren = _interopRequireDefault(require("./helpers/build-children"));

var _attributes = require("./helpers/attributes");

var _pluginSyntaxJsx = _interopRequireDefault(require("@babel/plugin-syntax-jsx"));

var messages = _interopRequireWildcard(require("./messages"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; if (obj != null) { var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function _default(_ref) {
  var t = _ref.types;
  var elementVisitor = {
    JSXNamespacedName: function JSXNamespacedName(path) {
      if (!this.opts.namespaceAttributes || path.parentPath.isJSXOpeningElement()) {
        throw path.buildCodeFrameError("JSX Namespaces aren't supported.");
      }
    },
    JSXElement: {
      enter: function enter(path) {
        this.elementVarsStack.push([]);
        var secondaryTree = !(0, _rootJsx.isRootOrDescendant)(path, this);
        var needsWrapper = secondaryTree || !(0, _completionRecords.isCompletionRecord)(path, this); // If this element needs to be wrapped in a closure, we need to transform
        // it's children without wrapping them.

        if (needsWrapper) {
          // If this element needs a closure wrapper, we need a new array of
          // closure parameters.
          this.closureVarsStack.push([]);
          path.traverse(_extractExpressions["default"], this);
          path.traverse(elementVisitor, this);
        }
      },
      exit: function exit(path) {
        var replacedElements = this.replacedElements;
        var secondaryTree = !(0, _rootJsx.isRootOrDescendant)(path, this);
        var needsWrapper = secondaryTree || !(0, _completionRecords.isCompletionRecord)(path, this);
        var _path = path,
            parentPath = _path.parentPath;
        var explicitReturn = parentPath.isReturnStatement();
        var implicitReturn = parentPath.isArrowFunctionExpression();
        var openingElementPath = path.get("openingElement");
        var openingElement = (0, _elementOpenCall["default"])(openingElementPath, this);
        var closingElement = (0, _elementCloseCall["default"])(openingElementPath, this);
        var elementVars = this.elementVarsStack.pop();
        var isSkipping = (0, _attributes.hasSkip)(openingElementPath.get("attributes"), this);
        var children = isSkipping ? [(0, _toFunctionCall["default"])((0, _idomMethod["default"])("skip", this), [])] : (0, _buildChildren["default"])(path.get("children"), this);
        var elements = [].concat(_toConsumableArray(elementVars), [openingElement], _toConsumableArray(children));

        if (closingElement) {
          elements.push(closingElement);
        } // Expressions Containers must contain an expression and not statements.
        // This will be flattened out into statements later.


        if (!needsWrapper && (implicitReturn || parentPath.isJSXExpressionContainer())) {
          var sequence = t.sequenceExpression(elements);
          path.replaceWith(sequence);
          replacedElements.add(path);
          return;
        }

        if (explicitReturn || implicitReturn || needsWrapper) {
          // Transform (recursively) any sequence expressions into a series of
          // statements.
          elements = (0, _flattenExpressions["default"])(elements); // Ensure the last statement returns the DOM element.

          elements = (0, _statementsWithReturnLast["default"])(elements);
        }

        if (needsWrapper) {
          // Create a wrapper around our element, and mark it as a one so later
          // child expressions can identify and "render" it.
          var closureVars = this.closureVarsStack.pop();
          var params = closureVars.map(function (e) {
            return e.id;
          });
          var wrapper = (0, _hoist.addHoistedDeclarator)(path.scope, (0, _hoist.generateHoistName)(path.get("openingElement"), "wrapper"), t.functionExpression(null, params, t.blockStatement(elements)), this); // If we're the direct child of a JSXAttribute, we have to wrap the wrapper
          // call in an expression container.

          if (parentPath.isJSXAttribute()) {
            path.replaceWith(t.jSXExpressionContainer(t.jSXEmptyExpression()));
            path = path.get("expression");
          }

          var args = [wrapper];

          if (closureVars.length) {
            var paramArgs = closureVars.map(function (e) {
              return e.init;
            });
            args.push(t.arrayExpression(paramArgs));
          }

          var wrapperCall = (0, _toFunctionCall["default"])((0, _jsxWrapper["default"])(this), args);
          replacedElements.add(path);
          path.replaceWith(wrapperCall);
          return;
        } // This is the main JSX element. Replace the return statement
        // with all the nested calls, returning the main JSX element.


        if (explicitReturn) {
          parentPath.replaceWithMultiple(elements);
          replacedElements.add(parentPath);
        } else {
          replacedElements.add(path);
          path.replaceWithMultiple(elements);
        }
      }
    }
  }; // This visitor first finds the root element, and ignores all the others.

  return {
    inherits: _pluginSyntaxJsx["default"],
    visitor: {
      Program: {
        enter: function enter(path) {
          if (this.opts.prefix) {
            throw new Error(messages.prefixOptionRemoved);
          }

          if (this.opts.runtime) {
            throw new Error(messages.runtimeOptionRemoved);
          }

          if (this.opts.inlineExpressions) {
            path.traverse(_inlineExpressions["default"], this);
          }

          (0, _inject.setupInjector)(this);
          (0, _hoist.setupHoists)(this);
        },
        exit: function exit(path) {
          (0, _hoist.hoist)(path, this);
          (0, _inject.injectHelpers)(this);
        }
      },
      JSXElement: {
        exit: function exit(path) {
          var parent = path.findParent(function (p) {
            return p.isFunction() || p.isProgram();
          });
          var root = (0, _rootJsx["default"])(parent, this);

          if (!root || root.getFunctionParent() === parent) {
            var state = {
              replacedElements: new WeakSet(),
              fastRoots: new WeakSet(),
              closureVarsStack: [],
              elementVarsStack: [],
              file: this.file,
              opts: this.opts
            };
            parent.traverse(elementVisitor, state);
          }
        }
      }
    }
  };
}